// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ChargedFinalState.hh"

namespace Rivet {

  class ATLAS_2016_I1426695 : public Analysis {
  public:

    //phase space regions
    enum regionID {
      k_pt100_nch2_eta25 = 0,
      k_pt500_nch1_eta25,
      k_pt500_nch6_eta25,
      k_pt500_nch20_eta25,
      k_pt500_nch50_eta25,
      kNregions
    };

    //nch cut for each region
    const static int nchCut[kNregions] ; 


    /// Helper for collectively filling Nch, pT, eta, and pT vs. Nch histograms
    void fillPtEtaNch(const ChargedFinalState& cfs, int iRegion, double weight) {

      // Get number of particles 
      int nch = 0;
      foreach (const Particle& p, cfs.particles())  {
        PdgId pdg = p.abspid ();
        if ( pdg == 3112 || // Sigma-
	           pdg == 3222 || // Sigma+
	           pdg == 3312 || // Xi-
	           pdg == 3334 )  // Omega-
	           continue;
	      nch++;
      }

      //skip if event fails multiplicity cut
      if (nch < nchCut[iRegion])  return;

      //fill event weight info
      _sumW[iRegion] += weight;

      // Fill nch
      if (iRegion == k_pt100_nch2_eta25 || iRegion == k_pt500_nch1_eta25) {
        _hist_nch[iRegion]->fill(nch, weight);
      }
      
      // Loop over particles, fill pT, eta and ptnch
      foreach (const Particle& p, cfs.particles())  {

        PdgId pdg = p.abspid ();
	      if ( pdg == 3112 || // Sigma-
             pdg == 3222 || // Sigma+
	           pdg == 3312 || // Xi-
	           pdg == 3334 )  // Omega-
	           continue;

        //here, we don't have strange particles anymore
        const double pt  = p.pT()/GeV;
        const double eta = p.eta();

        _hist_pt [iRegion]->fill(pt , weight/pt);
        _hist_eta[iRegion]->fill(eta, weight);

        if (iRegion == k_pt100_nch2_eta25 || iRegion == k_pt500_nch1_eta25) {
          _hist_ptnch[iRegion]->fill(nch, pt, weight);
        }
      } //end loop over particles
    }


    /// Default constructor
    ATLAS_2016_I1426695() : Analysis("ATLAS_2016_I1426695") {

      for (int iR=0; iR < kNregions; ++iR)  {
        _sumW[iR]  = 0.;
      }
    }

    /// Initialization, called once before running
    void init() {

      // Projections
      const ChargedFinalState cfs100_25(-2.5, 2.5, 100.0*MeV);
      addProjection(cfs100_25, "CFS100_25");

      const ChargedFinalState cfs500_25(-2.5, 2.5, 500.0*MeV);
      addProjection(cfs500_25, "CFS500_25");
      
      for (int iR=0; iR < kNregions; ++iR)  {
        if (iR == k_pt100_nch2_eta25 || iR == k_pt500_nch1_eta25) {
          _hist_nch  [iR] = bookHisto1D  (1, iR + 1, 1);
          _hist_ptnch[iR] = bookProfile1D(4, iR + 1, 1);
        }
        _hist_pt [iR] = bookHisto1D(2, iR + 1, 1);
        _hist_eta[iR] = bookHisto1D(3, iR + 1, 1);
      }
    }


    void analyze(const Event& event) {

      const double weight = event.weight();

      string fsName;

      for (int iR=0; iR < kNregions; ++iR)  {
	
        switch (iR) {
          case k_pt100_nch2_eta25  :  fsName = "CFS100_25"  ; break;
          case k_pt500_nch1_eta25  :  fsName = "CFS500_25"  ; break;
          case k_pt500_nch6_eta25  :  fsName = "CFS500_25"  ; break;
          case k_pt500_nch20_eta25 :  fsName = "CFS500_25"  ; break;
          case k_pt500_nch50_eta25 :  fsName = "CFS500_25"  ; break;
	      }
	
        const ChargedFinalState& cfs = applyProjection<ChargedFinalState>(event, fsName);

	      fillPtEtaNch(cfs, iR, weight);
      }
    }
    

    
    void finalize() {

      const double etaRangeSize = 2*2.5;
	
      //standard histograms
      for (int iR = 0; iR < kNregions; ++iR)  {
	
        if (_sumW[iR] > 0) {
          if (iR == k_pt100_nch2_eta25 || iR == k_pt500_nch1_eta25) {
            scale(_hist_nch[iR], 1.0/_sumW[iR]);
	        }
          scale(_hist_pt [iR], 1.0/_sumW[iR]/TWOPI/etaRangeSize);
          scale(_hist_eta[iR], 1.0/_sumW[iR]);
	      } 
      }
    }

    
  private:

    double _sumW[kNregions];

    Histo1DPtr   _hist_nch    [kNregions];
    Histo1DPtr   _hist_pt     [kNregions];
    Histo1DPtr   _hist_eta    [kNregions];
    Profile1DPtr _hist_ptnch  [kNregions];
  };

  //constants
  //cut on number of charged particles (excluding strange hadrons)
  const int ATLAS_2016_I1426695::nchCut[] = {2, 1, 6, 20, 50};

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2016_I1426695);

}
